@@ -15,16 +15,15 @@
 
 import abc
 
-import six
-
-from neutron.api import extensions
-from neutron.api.v2 import attributes as attr
-from neutron.api.v2 import resource_helper
-from neutron.plugins.common import constants as nconstants
-from neutron.services import service_base
 from neutron_lib.api import converters
-from neutron_lib.api import validators as validators
+from neutron_lib.api import extensions
+from neutron_lib.api import validators
+from neutron_lib.db import constants as db_const
 from neutron_lib import exceptions as nexception
+from neutron_lib.plugins import constants as nconstants
+from neutron_lib.services import base as service_base
+
+from neutron.api.v2 import resource_helper
 
 from neutron_vpnaas._i18n import _
 
@@ -47,6 +46,11 @@ class IPsecSiteConnectionMtuError(nexception.InvalidInput):
                 "for ipv%(version)s")
 
 
+class IPsecSiteConnectionPeerCidrError(nexception.InvalidInput):
+    message = _("ipsec_site_connection peer cidr %(peer_cidr)s is "
+                "invalid CIDR")
+
+
 class IKEPolicyNotFound(nexception.NotFound):
     message = _("IKEPolicy %(ikepolicy_id)s could not be found")
 
@@ -72,6 +76,11 @@ class SubnetInUseByEndpointGroup(nexception.InUse):
     message = _("Subnet %(subnet_id)s is used by endpoint group %(group_id)s")
 
 
+class SubnetInUseByIPsecSiteConnection(nexception.InUse):
+    message = _("Subnet %(subnet_id)s is used by ipsec site connection "
+                "%(ipsec_site_connection_id)s")
+
+
 class VPNStateInvalidToUpdate(nexception.BadRequest):
     message = _("Invalid state %(state)s of vpnaas resource %(id)s"
                 " for updating")
@@ -166,7 +175,7 @@ class EndpointGroupInUse(nexception.BadRequest):
 
 def _validate_subnet_list_or_none(data, key_specs=None):
     if data is not None:
-        validators.validate_subnet_list(data, key_specs)
+        return validators.validate_subnet_list(data, key_specs)
 
 validators.add_validator('type:subnet_list_or_none',
                         _validate_subnet_list_or_none)
@@ -184,8 +193,8 @@ vpn_supported_lifetime_units = ['seconds']
 vpn_supported_pfs = ['group2', 'group5', 'group14']
 vpn_supported_ike_versions = ['v1', 'v2']
 vpn_supported_auth_mode = ['psk']
-vpn_supported_auth_algorithms = ['sha1', 'sha256']
-vpn_supported_phase1_negotiation_mode = ['main']
+vpn_supported_auth_algorithms = ['sha1', 'sha256', 'sha384', 'sha512']
+vpn_supported_phase1_negotiation_mode = ['main', 'aggressive']
 
 vpn_lifetime_limits = (60, validators.UNLIMITED)
 positive_int = (0, validators.UNLIMITED)
@@ -196,23 +205,30 @@ RESOURCE_ATTRIBUTE_MAP = {
         'id': {'allow_post': False, 'allow_put': False,
                'validate': {'type:uuid': None},
                'is_visible': True,
-               'primary_key': True},
+               'primary_key': True,
+               'is_sort_key': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
-                      'validate': {'type:string': attr.TENANT_ID_MAX_LEN},
+                      'validate': {
+                          'type:string': db_const.PROJECT_ID_FIELD_SIZE},
                       'required_by_policy': True,
-                      'is_visible': True},
+                      'is_visible': True,
+                      'is_sort_key': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
-                 'is_visible': True, 'default': ''},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
+                 'is_visible': True, 'default': '',
+                 'is_sort_key': True},
         'description': {'allow_post': True, 'allow_put': True,
-                        'validate': {'type:string': attr.DESCRIPTION_MAX_LEN},
+                        'validate': {
+                            'type:string': db_const.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'subnet_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:uuid_or_none': None},
-                      'is_visible': True, 'default': None},
+                      'is_visible': True, 'default': None,
+                      'is_sort_key': True},
         'router_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:uuid': None},
-                      'is_visible': True},
+                      'is_visible': True,
+                      'is_sort_key': True},
         'admin_state_up': {'allow_post': True, 'allow_put': True,
                            'default': True,
                            'convert_to': converters.convert_to_boolean,
@@ -229,16 +245,21 @@ RESOURCE_ATTRIBUTE_MAP = {
         'id': {'allow_post': False, 'allow_put': False,
                'validate': {'type:uuid': None},
                'is_visible': True,
-               'primary_key': True},
+               'primary_key': True,
+               'is_sort_key': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
-                      'validate': {'type:string': attr.TENANT_ID_MAX_LEN},
+                      'validate': {
+                          'type:string': db_const.PROJECT_ID_FIELD_SIZE},
                       'required_by_policy': True,
-                      'is_visible': True},
+                      'is_visible': True,
+                      'is_sort_key': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
-                 'is_visible': True, 'default': ''},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
+                 'is_visible': True, 'default': '',
+                 'is_sort_key': True},
         'description': {'allow_post': True, 'allow_put': True,
-                        'validate': {'type:string': attr.DESCRIPTION_MAX_LEN},
+                        'validate': {
+                            'type:string': db_const.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'local_id': {'allow_post': True, 'allow_put': True,
                     'validate': {'type:string': None},
@@ -315,16 +336,21 @@ RESOURCE_ATTRIBUTE_MAP = {
         'id': {'allow_post': False, 'allow_put': False,
                'validate': {'type:uuid': None},
                'is_visible': True,
-               'primary_key': True},
+               'primary_key': True,
+               'is_sort_key': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
-                      'validate': {'type:string': attr.TENANT_ID_MAX_LEN},
+                      'validate': {
+                          'type:string': db_const.PROJECT_ID_FIELD_SIZE},
                       'required_by_policy': True,
-                      'is_visible': True},
+                      'is_visible': True,
+                      'is_sort_key': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
-                 'is_visible': True, 'default': ''},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
+                 'is_visible': True, 'default': '',
+                 'is_sort_key': True},
         'description': {'allow_post': True, 'allow_put': True,
-                        'validate': {'type:string': attr.DESCRIPTION_MAX_LEN},
+                        'validate': {
+                            'type:string': db_const.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'transform_protocol': {
             'allow_post': True,
@@ -379,16 +405,21 @@ RESOURCE_ATTRIBUTE_MAP = {
         'id': {'allow_post': False, 'allow_put': False,
                'validate': {'type:uuid': None},
                'is_visible': True,
-               'primary_key': True},
+               'primary_key': True,
+               'is_sort_key': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
-                      'validate': {'type:string': attr.TENANT_ID_MAX_LEN},
+                      'validate': {
+                          'type:string': db_const.PROJECT_ID_FIELD_SIZE},
                       'required_by_policy': True,
-                      'is_visible': True},
+                      'is_visible': True,
+                      'is_sort_key': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
-                 'is_visible': True, 'default': ''},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
+                 'is_visible': True, 'default': '',
+                 'is_sort_key': True},
         'description': {'allow_post': True, 'allow_put': True,
-                        'validate': {'type:string': attr.DESCRIPTION_MAX_LEN},
+                        'validate': {
+                            'type:string': db_const.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'auth_algorithm': {'allow_post': True, 'allow_put': True,
                            'default': 'sha1',
@@ -461,7 +492,6 @@ class Vpnaas(extensions.ExtensionDescriptor):
         plural_mappings = resource_helper.build_plural_mappings(
             special_mappings, RESOURCE_ATTRIBUTE_MAP)
         plural_mappings['peer_cidrs'] = 'peer_cidr'
-        attr.PLURALS.update(plural_mappings)
         return resource_helper.build_resource_info(plural_mappings,
                                                    RESOURCE_ATTRIBUTE_MAP,
                                                    nconstants.VPN,
@@ -479,15 +509,10 @@ class Vpnaas(extensions.ExtensionDescriptor):
     def get_extended_resources(self, version):
         if version == "2.0":
             return RESOURCE_ATTRIBUTE_MAP
-        else:
-            return {}
+        return {}
 
 
-@six.add_metaclass(abc.ABCMeta)
-class VPNPluginBase(service_base.ServicePluginBase):
-
-    def get_plugin_name(self):
-        return nconstants.VPN
+class VPNPluginBase(service_base.ServicePluginBase, metaclass=abc.ABCMeta):
 
     def get_plugin_type(self):
         return nconstants.VPN
