@@ -17,8 +17,8 @@ import difflib
 import io
 import os
 import socket
+from unittest import mock
 
-import mock
 import netaddr
 from neutron.agent.l3 import dvr_edge_router
 from neutron.agent.l3 import dvr_snat_ns
@@ -73,6 +73,7 @@ FAKE_VPN_SERVICE = {
          'admin_state_up': True,
          'id': FAKE_IPSEC_SITE_CONNECTION1_ID,
          'external_ip': '60.0.0.4',
+         'local_id': '60.0.0.4',
          'peer_address': '60.0.0.5',
          'mtu': 1500,
          'peer_id': '60.0.0.5',
@@ -90,6 +91,7 @@ FAKE_VPN_SERVICE = {
          'local_ip_vers': 4,
          'admin_state_up': True,
          'external_ip': '60.0.0.4',
+         'local_id': '60.0.0.4',
          'peer_address': '60.0.0.6',
          'peer_id': '60.0.0.6',
          'mtu': 1500,
@@ -170,7 +172,7 @@ conn %%default
 conn %(conn1_id)s
     %(next_hop)s
     left=%(left)s
-    leftid=%(left)s
+    leftid=%(leftid)s
     auto=start
     # NOTE:REQUIRED
     # [subnet]
@@ -192,7 +194,7 @@ conn %(conn1_id)s
     %(conn_details)sconn %(conn2_id)s
     %(next_hop)s
     left=%(left)s
-    leftid=%(left)s
+    leftid=%(leftid)s
     auto=start
     # NOTE:REQUIRED
     # [subnet]
@@ -238,7 +240,7 @@ conn %(conn1_id)s
     keyexchange=ikev1
     left=%(left)s
     leftsubnet=%(local_cidrs1)s
-    leftid=%(left)s
+    leftid=%(leftid)s
     leftfirewall=yes
     right=%(right1)s
     rightsubnet=%(peer_cidrs1)s
@@ -257,7 +259,7 @@ conn %(conn2_id)s
     keyexchange=ikev1
     left=%(left)s
     leftsubnet=%(local_cidrs2)s
-    leftid=%(left)s
+    leftid=%(leftid)s
     leftfirewall=yes
     right=%(right2)s
     rightsubnet=%(peer_cidrs2)s
@@ -292,12 +294,19 @@ EXPECTED_IPSEC_STRONGSWAN_SECRET_CONF = '''
 ''' % FAKE_VPNSERVICE_ID
 
 PLUTO_ACTIVE_STATUS = """000 "%(conn_id)s/0x1": erouted;\n
-000 #4: "%(conn_id)s/0x1":500 STATE_QUICK_R2 (IPsec SA established);""" % {
+000 #4: "%(conn_id)s/0x1":500 STATE_QUICK_R2 (IPsec SA established); \
+newest IPSEC;""" % {
+    'conn_id': FAKE_IPSEC_SITE_CONNECTION2_ID}
+PLUTO_ACTIVE_STATUS_IKEV2 = """000 "%(conn_id)s/0x1": erouted;\n
+000 #4: "%(conn_id)s/0x1":500 STATE_PARENT_R2 (PARENT SA established); \
+newest IPSEC;""" % {
     'conn_id': FAKE_IPSEC_SITE_CONNECTION2_ID}
 PLUTO_MULTIPLE_SUBNETS_ESTABLISHED_STATUS = """000 "%(conn_id1)s/1x1": erouted;\n
-000 #4: "%(conn_id1)s/1x1":500 STATE_QUICK_R2 (IPsec SA established);\n
+000 #4: "%(conn_id1)s/1x1":500 STATE_QUICK_R2 (IPsec SA established); \
+newest IPSEC;\n
 000 "%(conn_id2)s/2x1": erouted;\n
-000 #4: "%(conn_id2)s/2x1":500 STATE_QUICK_R2 (IPsec SA established);\n""" % {
+000 #4: "%(conn_id2)s/2x1":500 STATE_QUICK_R2 (IPsec SA established); \
+newest IPSEC;\n""" % {
     'conn_id1': FAKE_IPSEC_SITE_CONNECTION1_ID,
     'conn_id2': FAKE_IPSEC_SITE_CONNECTION2_ID}
 PLUTO_ACTIVE_NO_IPSEC_SA_STATUS = """000 "%(conn_id)s/0x1": erouted;\n
@@ -320,7 +329,7 @@ class BaseIPsecDeviceDriver(base.BaseTestCase):
               vpnservice=FAKE_VPN_SERVICE):
         super(BaseIPsecDeviceDriver, self).setUp()
         for klass in [
-            'neutron.common.rpc.Connection',
+            'neutron_lib.rpc.Connection',
             'oslo_service.loopingcall.FixedIntervalLoopingCall'
         ]:
             mock.patch(klass).start()
@@ -386,14 +395,13 @@ class BaseIPsecDeviceDriver(base.BaseTestCase):
                 conn['peer_address'] = peers[i]
 
         local_ip = overrides.get('local')
-        if local_ip:
-            for conn in self.vpnservice['ipsec_site_connections']:
-                conn['external_ip'] = local_ip
-
         local_id = overrides.get('local_id')
         if local_ip:
             for conn in self.vpnservice['ipsec_site_connections']:
-                conn['local_id'] = local_id
+                conn['external_ip'] = local_ip
+                conn['local_id'] = local_ip
+                if local_id:
+                    conn['local_id'] = local_id
 
     def check_config_file(self, expected, actual):
         expected = expected.strip()
@@ -798,6 +806,29 @@ class IPSecDeviceLegacy(BaseIPsecDeviceDriver):
         self.assertEqual(constants.ACTIVE,
                          ipsec_site_conn[connection_id]['status'])
 
+    def _test_status_handling_for_ike_v2_active_connection(self,
+            active_status):
+        """Test status handling for active connection."""
+        router_id = self.router.router_id
+        connection_id = FAKE_IPSEC_SITE_CONNECTION2_ID
+        ike_policy = {'ike_version': 'v2',
+                      'encryption_algorithm': 'aes-128',
+                      'auth_algorithm': 'sha1',
+                      'pfs': 'group5',
+                      'lifetime_value': 3600}
+        vpn_service = FAKE_VPN_SERVICE
+        for isc in vpn_service["ipsec_site_connections"]:
+            isc['ikepolicy'] = ike_policy
+        self.driver.ensure_process(router_id, vpn_service)
+        self._execute.return_value = active_status
+        self.driver.report_status(mock.Mock())
+        process_status = self.driver.process_status_cache[
+            router_id]
+        ipsec_site_conn = process_status['ipsec_site_connections']
+        self.assertEqual(constants.ACTIVE, process_status['status'])
+        self.assertEqual(constants.ACTIVE,
+                         ipsec_site_conn[connection_id]['status'])
+
     def _test_connection_names_handling_for_multiple_subnets(self,
                                                              active_status):
         """Test connection names handling for multiple subnets."""
@@ -903,6 +934,27 @@ class IPSecDeviceDVR(BaseIPsecDeviceDriver):
         router.snat_iptables_manager.apply = self.apply_mock
         self.driver.routers[FAKE_ROUTER_ID] = router
 
+    def test_sync_dvr(self):
+        fake_vpn_service = FAKE_VPN_SERVICE
+        self.driver.agent_rpc.get_vpn_services_on_host.return_value = [
+            fake_vpn_service]
+        context = mock.Mock()
+        self.driver._sync_vpn_processes = mock.Mock()
+        self.driver._delete_vpn_processes = mock.Mock()
+        self.driver._cleanup_stale_vpn_processes = mock.Mock()
+        sync_routers = [{'id': fake_vpn_service['router_id']}]
+        sync_router_ids = [fake_vpn_service['router_id']]
+        with mock.patch.object(self.driver,
+                'get_process_status_cache') as process_status:
+            self.driver.sync(context, sync_routers)
+            self.driver._sync_vpn_processes.assert_called_once_with(
+                [fake_vpn_service], sync_router_ids)
+            self.driver._delete_vpn_processes.assert_called_once_with(
+                sync_router_ids, sync_router_ids)
+            self.driver._cleanup_stale_vpn_processes.assert_called_once_with(
+                sync_router_ids)
+            self.assertEqual(0, process_status.call_count)
+
     def test_get_namespace_for_dvr_edge_router(self):
         namespace = self.driver.get_namespace(FAKE_ROUTER_ID)
         self.assertEqual('snat-' + FAKE_ROUTER_ID, namespace)
@@ -983,6 +1035,10 @@ class TestOpenSwanConfigGeneration(BaseIPsecDeviceDriver):
                 virtual_privates.append('%%v%s:%s' % (version, net))
         peer_cidrs = [' '.join(cidr) for cidr in cidrs]
         local_ip = info.get('local', '60.0.0.4')
+        local_id = info.get('local_id')
+        leftid = local_ip
+        if local_id:
+            leftid = local_id
         version = info.get('local_ip_vers', 4)
         next_hop = IPV4_NEXT_HOP if version == 4 else IPV6_NEXT_HOP % local_ip
         peer_ips = info.get('peers', ['60.0.0.5', '60.0.0.6'])
@@ -995,6 +1051,7 @@ class TestOpenSwanConfigGeneration(BaseIPsecDeviceDriver):
             'local_ver': version,
             'peer_cidrs1': peer_cidrs[0], 'peer_cidrs2': peer_cidrs[1],
             'left': local_ip,
+            'leftid': leftid,
             'right1': peer_ips[0], 'right2': peer_ips[1],
             'conn1_id': FAKE_IPSEC_SITE_CONNECTION1_ID,
             'conn2_id': FAKE_IPSEC_SITE_CONNECTION2_ID,
@@ -1038,6 +1095,18 @@ class TestOpenSwanConfigGeneration(BaseIPsecDeviceDriver):
         self.process.update_vpnservice(self.vpnservice)
         self._test_ipsec_connection_config(overrides)
 
+    def test_config_files_with_ipv6_addresses_without_local_id(self):
+        """Test creating config files using IPv6 addressing."""
+        overrides = {'local_cidrs': [['2002:0a00::/48'], ['2002:0b00::/48']],
+                     'local_ip_vers': 6,
+                     'peer_cidrs': [['2002:1400::/48', '2002:1e00::/48'],
+                                    ['2002:2800::/48', '2002:3200::/48']],
+                     'local': '2002:3c00:0004::',
+                     'peers': ['2002:3c00:0005::', '2002:3c00:0006::']}
+        self.modify_config_for_test(overrides)
+        self.process.update_vpnservice(self.vpnservice)
+        self._test_ipsec_connection_config(overrides)
+
     def test_secrets_config_file(self):
         expected = EXPECTED_IPSEC_OPENSWAN_SECRET_CONF
         actual = self.process._gen_config_content(
@@ -1067,6 +1136,10 @@ class IPsecStrongswanConfigGeneration(BaseIPsecDeviceDriver):
                                         ['40.0.0.0/24', '50.0.0.0/24']])
         peer_cidrs = [','.join(cidr) for cidr in cidrs]
         local_ip = info.get('local', '60.0.0.4')
+        local_id = info.get('local_id')
+        leftid = local_ip
+        if local_id:
+            leftid = local_id
         peer_ips = info.get('peers', ['60.0.0.5', '60.0.0.6'])
         auth_mode = info.get('ipsec_auth', STRONGSWAN_AUTH_ESP)
         return EXPECTED_IPSEC_STRONGSWAN_CONF % {
@@ -1074,6 +1147,7 @@ class IPsecStrongswanConfigGeneration(BaseIPsecDeviceDriver):
             'local_cidrs1': local_cidrs[0], 'local_cidrs2': local_cidrs[1],
             'peer_cidrs1': peer_cidrs[0], 'peer_cidrs2': peer_cidrs[1],
             'left': local_ip,
+            'leftid': leftid,
             'right1': peer_ips[0], 'right2': peer_ips[1],
             'dpd_action': 'hold',
             'dpd_delay': 30,
@@ -1109,6 +1183,16 @@ class IPsecStrongswanConfigGeneration(BaseIPsecDeviceDriver):
         self.process.update_vpnservice(self.vpnservice)
         self._test_ipsec_connection_config(overrides)
 
+    def test_ipsec_config_file_for_v6_without_local_id(self):
+        overrides = {'local_cidrs': [['2002:0a00::/48'], ['2002:0b00::/48']],
+                     'peer_cidrs': [['2002:1400::/48', '2002:1e00::/48'],
+                                    ['2002:2800::/48', '2002:3200::/48']],
+                     'local': '2002:3c00:0004::',
+                     'peers': ['2002:3c00:0005::', '2002:3c00:0006::']}
+        self.modify_config_for_test(overrides)
+        self.process.update_vpnservice(self.vpnservice)
+        self._test_ipsec_connection_config(overrides)
+
     def test_strongswan_default_config_file(self):
         expected = EXPECTED_STRONGSWAN_DEFAULT_CONF
         actual = self.process._gen_config_content(
@@ -1394,6 +1478,11 @@ class TestOpenSwanProcess(IPSecDeviceLegacy):
         """Test status handling for active connection."""
         self._test_status_handling_for_active_connection(PLUTO_ACTIVE_STATUS)
 
+    def test_status_handling_for_ike_v2_active_connection(self):
+        """Test status handling for active connection."""
+        self._test_status_handling_for_ike_v2_active_connection(
+            PLUTO_ACTIVE_STATUS_IKEV2)
+
     def test_status_handling_for_deleted_connection(self):
         """Test status handling for deleted connection."""
         self._test_status_handling_for_deleted_connection(NOT_RUNNING_STATUS)
